// HaxePreD: Haxe Preprocessor In D
// Written in the D programming language

module haxepred.cmd;

import tango.io.Stdout;

import semitwist.cmdlineparser;
import semitwist.util.all;

//TODO? If neither /o nor /so is specified, automatically assume "/o:infile{.preproc.hx}"

class CmdSwitches
{
	mixin(getter!(bool, "shouldExit"));

	public this(char[][] args)
	{
		init();
		shouldExit = parse(args);
	}
	
	// Direct command-line args
	mixin(getter!(bool, "help"));
	mixin(getter!(bool, "morehelp"));

	mixin(getter!(bool, "useStdin"));
	mixin(getter!(bool, "useStdout"));

	mixin(getter!(char[], "inFilename"));
	mixin(getter!(char[], "outFilename"));

	mixin(getter!(char[][], "defines"));

	// Private ------
	private CmdLineParser cmd;
	private void init()
	{
		cmd = new CmdLineParser();
		mixin(defineArg!(cmd, "help",     _help,        ArgFlag.Optional, "Displays a help summary and exits" ));
		mixin(defineArg!(cmd, "morehelp", _morehelp,    ArgFlag.Optional, "Displays a detailed help message and exits" ));
		mixin(defineArg!(cmd, "si",       _useStdin,    ArgFlag.Optional, "Get input from stdin" ));
		mixin(defineArg!(cmd, "so",       _useStdout,   ArgFlag.Optional, "Send output to stdout" ));
		mixin(defineArg!(cmd, "i",        _inFilename,  ArgFlag.Optional|ArgFlag.Switchless, "Input filename" ));
		mixin(defineArg!(cmd, "o",        _outFilename, ArgFlag.Optional, "Output filename" ));
		mixin(defineArg!(cmd, "d",        _defines,     ArgFlag.Optional, "Define identifier" ));
	}
	
	version(Windows)
	{
		const char[] sampleUsageMsg = 
"Sample Usage:
    haxepred myfile.hx /o:preprocessed.hx /d:php /d:DEBUG
    haxepred myfile.hx /so > preprocessed.hx
    haxepred /si /so < myfile.hx > preprocessed.hx
";

	const char[] infoMsg = "
You must specify either an input file or /si (but not both).
You must specify either an output file or /so (or both).";
	}
	else
	{
		const char[] sampleUsageMsg = 
"Sample Usage:
    haxepred myfile.hx -o:preprocessed.hx -d:php -d:DEBUG
    haxepred myfile.hx --so | preprocessed.hx
    cat myfile.hx | haxepred --si --so | preprocessed.hx
";

	const char[] infoMsg = "
You must specify either an input file or --si (but not both).
You must specify either an output file or --so (or both).";
	}

	// Returns: Failure?
	private bool parse(char[][] args)
	{
		bool failure = false;
		
		void fail(char[] msg)
		{
			Stderr(msg).newline.newline;
			failure = true;
		}
		
		cmd.parse(args);
		if(morehelp)
		{
			Stderr(cmd.errorMsg).newline;
			Stdout(sampleUsageMsg).newline;
			Stdout(cmd.getDetailedUsage());
			Stdout(infoMsg).newline;
			return false;
		}
		
		if(!cmd.success || help)
			fail(cmd.errorMsg);
		else
		{
			if(!useStdin && inFilename=="")
				fail("You did not specify an input");
			
			if(useStdin && inFilename!="")
				fail("You cannot set input to both stdin and a file.");
			
			if(!useStdout && outFilename=="")
				fail("You did not specify an output");

			if(useStdout && outFilename!="")
				fail("You cannot set output to both stdout and a file.");
		}
		
		if(failure)
		{
			Stdout(sampleUsageMsg).newline;
			Stdout(cmd.getUsage(16));
			Stdout(infoMsg).newline;
		}

		return failure;
	}
	
}
